"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Master = void 0;
const file_helper_1 = require("../helper/file-helper");
const xml_helper_1 = require("../helper/xml-helper");
const element_type_1 = require("../enums/element-type");
const image_1 = require("../shapes/image");
const chart_1 = require("../shapes/chart");
const generic_1 = require("../shapes/generic");
const xml_relationship_helper_1 = require("../helper/xml-relationship-helper");
const has_shapes_1 = __importDefault(require("./has-shapes"));
class Master extends has_shapes_1.default {
    constructor(params) {
        super();
        this.targetType = 'slideMaster';
        this.sourceTemplate = params.template;
        // ToDo analogue for slideMasters
        // this.sourceNumber = this.getSlideNumber(
        //   params.template,
        //   params.sourceIdentifier,
        // );
        this.sourceNumber = Number(params.sourceIdentifier);
        this.sourcePath = `ppt/slideMasters/slideMaster${this.sourceNumber}.xml`;
        this.relsPath = `ppt/slideMasters/_rels/slideMaster${this.sourceNumber}.xml.rels`;
        this.modifications = [];
        this.importElements = [];
        this.status = params.presentation.status;
        this.content = params.presentation.content;
    }
    /**
     * ToDo: Implement creationIds as well for slideMasters
     *
     * Try to convert a given slide's creationId to corresponding slide number.
     * Used if automizer is run with useCreationIds: true
     * @internal
     * @param PresTemplate
     * @slideNumber SourceSlideIdentifier
     * @returns number
     */
    getSlideNumber(template, slideIdentifier) {
        if (template.useCreationIds === true &&
            template.creationIds !== undefined) {
            const matchCreationId = template.creationIds.find((slideInfo) => slideInfo.id === Number(slideIdentifier));
            if (matchCreationId) {
                return matchCreationId.number;
            }
            throw ('Could not find slide number for creationId: ' +
                slideIdentifier +
                '@' +
                template.name);
        }
        return slideIdentifier;
    }
    /**
     * Appends slide
     * @internal
     * @param targetTemplate
     * @returns append
     */
    append(targetTemplate) {
        return __awaiter(this, void 0, void 0, function* () {
            this.targetTemplate = targetTemplate;
            this.targetArchive = yield targetTemplate.archive;
            this.targetNumber = targetTemplate.incrementCounter('masters');
            this.targetPath = `ppt/slideMasters/slideMaster${this.targetNumber}.xml`;
            this.targetRelsPath = `ppt/slideMasters/_rels/slideMaster${this.targetNumber}.xml.rels`;
            this.sourceArchive = yield this.sourceTemplate.archive;
            console.log('Importing slideMaster ' + this.targetNumber);
            yield this.copySlideFiles();
            yield this.copyRelatedLayouts();
            yield this.copyRelatedContent();
            yield this.addSlideMasterToPresentation();
            yield this.copyThemeFiles();
            if (this.importElements.length) {
                yield this.importedSelectedElements();
            }
            yield this.applyModifications();
            yield this.cleanSlide(this.targetPath);
            yield this.checkIntegrity();
        });
    }
    copyRelatedLayouts() {
        return __awaiter(this, void 0, void 0, function* () {
            const masterToLayout = yield new xml_relationship_helper_1.XmlRelationshipHelper().initialize(this.targetArchive, `slideMaster${this.targetNumber}.xml.rels`, `ppt/slideMasters/_rels`);
            const targets = masterToLayout.getTargetsByPrefix('../slideLayouts/slideLayout');
            for (const target of targets) {
                const nextSlideLayoutNumber = this.targetTemplate.incrementCounter('layouts');
                this.targetTemplate.mapContents('slideLayout', this.sourceTemplate.name, target.number, nextSlideLayoutNumber);
                target.updateTargetIndex(nextSlideLayoutNumber);
                yield file_helper_1.FileHelper.zipCopyWithRelations(this, 'slideLayout', target.number, nextSlideLayoutNumber);
                const layoutTargetHelper = yield new xml_relationship_helper_1.XmlRelationshipHelper().initialize(this.targetArchive, `slideLayout${nextSlideLayoutNumber}.xml.rels`, `ppt/slideLayouts/_rels`);
                const layoutToMaster = layoutTargetHelper.getTargetsByPrefix('../slideMasters/slideMaster');
                layoutToMaster[0].updateTargetIndex(this.targetNumber);
                yield layoutTargetHelper.assertRelatedContent(this.sourceArchive, false, true);
                yield this.cleanSlide(`ppt/slideLayouts/slideLayout${nextSlideLayoutNumber}.xml`);
                yield this.appendSlideLayoutToContentType(this.targetArchive, nextSlideLayoutNumber);
            }
            return targets;
        });
    }
    copyThemeFiles() {
        return __awaiter(this, void 0, void 0, function* () {
            const targets = yield xml_helper_1.XmlHelper.getRelationshipTargetsByPrefix(this.targetArchive, `ppt/slideMasters/_rels/slideMaster${this.targetNumber}.xml.rels`, '../theme/theme');
            if (!targets.length) {
                return;
            }
            const themeTarget = targets[0];
            const themeSourceId = themeTarget.number;
            const themeTargetId = this.targetTemplate.incrementCounter('themes');
            yield file_helper_1.FileHelper.zipCopy(this.sourceArchive, `ppt/theme/theme${themeSourceId}.xml`, this.targetArchive, `ppt/theme/theme${themeTargetId}.xml`);
            yield this.appendThemeToContentType(this.targetArchive, themeTargetId);
            yield xml_helper_1.XmlHelper.replaceAttribute(this.targetArchive, `ppt/slideMasters/_rels/slideMaster${this.targetNumber}.xml.rels`, 'Relationship', 'Id', themeTarget.rId, `../theme/theme${themeTargetId}.xml`, 'Target');
        });
    }
    /**
     * Modifies slide
     * @internal
     * @param callback
     */
    modify(callback) {
        this.modifications.push(callback);
    }
    /**
     * Adds slide to presentation
     * @internal
     * @returns slide to presentation
     */
    addSlideMasterToPresentation() {
        return __awaiter(this, void 0, void 0, function* () {
            const relId = yield xml_helper_1.XmlHelper.getNextRelId(this.targetArchive, 'ppt/_rels/presentation.xml.rels');
            yield this.appendToSlideRel(this.targetArchive, relId, this.targetNumber);
            yield this.appendToSlideList(this.targetArchive, relId);
            yield this.appendSlideMasterToContentType(this.targetArchive, this.targetNumber);
        });
    }
    /**
     * ToDo: This equals the corresponding method in slide.ts
     *
     * Select and modify a single element on an added slide.
     * @param {string} selector - Element's name on the slide.
     * Should be a unique string defined on the "Selection"-pane within ppt.
     * @param {ShapeModificationCallback | ShapeModificationCallback[]} callback - One or more callback functions to apply.
     * Depending on the shape type (e.g. chart or table), different arguments will be passed to the callback.
     */
    modifyElement(selector, callback) {
        const presName = this.sourceTemplate.name;
        const slideNumber = this.sourceNumber;
        return this.addElementToModificationsList(presName, slideNumber, selector, 'modify', callback);
    }
    /**
     * ToDo: This equals the corresponding method in slide.ts
     *
     * Select, insert and (optionally) modify a single element to a slide.
     * @param {string} presName - Filename or alias name of the template presentation.
     * Must have been importet with Automizer.load().
     * @param {number} slideNumber - Slide number within the specified template to search for the required element.
     * @param {ShapeModificationCallback | ShapeModificationCallback[]} callback - One or more callback functions to apply.
     * Depending on the shape type (e.g. chart or table), different arguments will be passed to the callback.
     */
    addElement(presName, slideNumber, selector, callback) {
        return this.addElementToModificationsList(presName, slideNumber, selector, 'append', callback);
    }
    /**
     * ToDo: This equals the corresponding method in slide.ts
     *
     * Remove a single element from slide.
     * @param {string} selector - Element's name on the slide.
     */
    removeElement(selector) {
        const presName = this.sourceTemplate.name;
        const slideNumber = this.sourceNumber;
        return this.addElementToModificationsList(presName, slideNumber, selector, 'remove', undefined);
    }
    /**
     * ToDo: This equals the corresponding method in slide.ts
     *
     * Adds element to modifications list
     * @internal
     * @param presName
     * @param slideNumber
     * @param selector
     * @param mode
     * @param [callback]
     * @returns element to modifications list
     */
    addElementToModificationsList(presName, slideNumber, selector, mode, callback) {
        this.importElements.push({
            presName,
            slideNumber,
            selector,
            mode,
            callback,
        });
        return this;
    }
    /**
     * ToDo: This equals the corresponding method in slide.ts
     *
     * Imported selected elements
     * @internal
     * @returns selected elements
     */
    importedSelectedElements() {
        return __awaiter(this, void 0, void 0, function* () {
            for (const element of this.importElements) {
                const info = yield this.getElementInfo(element);
                switch (info === null || info === void 0 ? void 0 : info.type) {
                    case element_type_1.ElementType.Chart:
                        yield new chart_1.Chart(info, this.targetType)[info.mode](this.targetTemplate, this.targetNumber, 'slideMaster');
                        break;
                    case element_type_1.ElementType.Image:
                        yield new image_1.Image(info, this.targetType)[info.mode](this.targetTemplate, this.targetNumber, 'slideMaster');
                        break;
                    case element_type_1.ElementType.Shape:
                        yield new generic_1.GenericShape(info, this.targetType)[info.mode](this.targetTemplate, this.targetNumber, 'slideMaster');
                        break;
                    default:
                        break;
                }
            }
        });
    }
    /**
     * ToDo: This *ALMOST* equals the corresponding method in slide.ts
     *
     * Gets element info
     * @internal
     * @param importElement
     * @returns element info
     */
    getElementInfo(importElement) {
        return __awaiter(this, void 0, void 0, function* () {
            const template = this.root.getTemplate(importElement.presName);
            const slideNumber = importElement.mode === 'append'
                ? this.getSlideNumber(template, importElement.slideNumber)
                : importElement.slideNumber;
            // It is possible to import shapes from loaded presentations,
            // as well as to modify an existing shape on current slideMaster
            const sourcePath = importElement.mode === 'append'
                ? `ppt/slides/slide${slideNumber}.xml`
                : `ppt/slideMasters/slideMaster${slideNumber}.xml`;
            // const sourcePath = `ppt/slideMasters/slideMaster${slideNumber}.xml`;
            // const sourcePath = `ppt/slides/slide${slideNumber}.xml`;
            const sourceArchive = yield template.archive;
            const useCreationIds = template.useCreationIds === true && template.creationIds !== undefined;
            const { sourceElement, selector } = yield this.findElementOnSlide(importElement.selector, sourceArchive, sourcePath, useCreationIds);
            if (!sourceElement) {
                console.error(`Can't find element on ${sourcePath} in ${importElement.presName}: `);
                console.log(importElement);
                return;
            }
            const appendElementParams = yield this.analyzeElement(sourceElement, sourceArchive, slideNumber);
            return {
                mode: importElement.mode,
                name: selector,
                hasCreationId: useCreationIds,
                sourceArchive,
                sourceSlideNumber: slideNumber,
                sourceElement,
                callback: importElement.callback,
                target: appendElementParams.target,
                type: appendElementParams.type,
            };
        });
    }
    /**
     * ToDo: This equals the corresponding method in slide.ts
     *
     * @param selector
     * @param sourceArchive
     * @param sourcePath
     * @param useCreationIds
     */
    findElementOnSlide(selector, sourceArchive, sourcePath, useCreationIds) {
        return __awaiter(this, void 0, void 0, function* () {
            const strategies = [];
            if (typeof selector === 'string') {
                if (useCreationIds) {
                    strategies.push({
                        mode: 'findByElementCreationId',
                        selector: selector,
                    });
                }
                strategies.push({
                    mode: 'findByElementName',
                    selector: selector,
                });
            }
            else if (selector.name) {
                strategies.push({
                    mode: 'findByElementCreationId',
                    selector: selector.creationId,
                });
                strategies.push({
                    mode: 'findByElementName',
                    selector: selector.name,
                });
            }
            for (const findElement of strategies) {
                const mode = findElement.mode;
                const sourceElement = yield xml_helper_1.XmlHelper[mode](sourceArchive, sourcePath, findElement.selector);
                if (sourceElement) {
                    return { sourceElement, selector: findElement.selector };
                }
            }
            return { sourceElement: undefined, selector: JSON.stringify(selector) };
        });
    }
    /**
     * ToDo: This equals the corresponding method in slide.ts
     *
     * Analyzes element
     * @internal
     * @param sourceElement
     * @param sourceArchive
     * @param slideNumber
     * @returns element
     */
    analyzeElement(sourceElement, sourceArchive, slideNumber) {
        return __awaiter(this, void 0, void 0, function* () {
            const isChart = sourceElement.getElementsByTagName('c:chart');
            if (isChart.length) {
                const target = yield xml_helper_1.XmlHelper.getTargetByRelId(sourceArchive, slideNumber, sourceElement, 'chart');
                return {
                    type: element_type_1.ElementType.Chart,
                    target: target,
                };
            }
            const isChartEx = sourceElement.getElementsByTagName('cx:chart');
            if (isChartEx.length) {
                const target = yield xml_helper_1.XmlHelper.getTargetByRelId(sourceArchive, slideNumber, sourceElement, 'chartEx');
                return {
                    type: element_type_1.ElementType.Chart,
                    target: target,
                };
            }
            const isImage = sourceElement.getElementsByTagName('p:nvPicPr');
            if (isImage.length) {
                return {
                    type: element_type_1.ElementType.Image,
                    target: yield xml_helper_1.XmlHelper.getTargetByRelId(sourceArchive, slideNumber, sourceElement, 'image'),
                };
            }
            return {
                type: element_type_1.ElementType.Shape,
            };
        });
    }
    /**
     * ToDo: This equals the corresponding method in slide.ts
     *
     * Applys modifications
     * @internal
     * @returns modifications
     */
    applyModifications() {
        return __awaiter(this, void 0, void 0, function* () {
            for (const modification of this.modifications) {
                const xml = yield xml_helper_1.XmlHelper.getXmlFromArchive(this.targetArchive, this.targetPath);
                modification(xml);
                xml_helper_1.XmlHelper.writeXmlToArchive(this.targetArchive, this.targetPath, xml);
            }
        });
    }
    /**
     * ToDo: This *almost* equals the corresponding method in slide.ts
     *
     * Removes all unsupported tags from slide xml.
     * E.g. added relations & tags by Thinkcell cannot
     * be processed by pptx-automizer at the moment.
     * @internal
     */
    cleanSlide(targetPath) {
        return __awaiter(this, void 0, void 0, function* () {
            const xml = yield xml_helper_1.XmlHelper.getXmlFromArchive(this.targetArchive, targetPath);
            this.unsupportedTags.forEach((tag) => {
                const drop = xml.getElementsByTagName(tag);
                const length = drop.length;
                if (length && length > 0) {
                    xml_helper_1.XmlHelper.sliceCollection(drop, 0);
                }
            });
            xml_helper_1.XmlHelper.writeXmlToArchive(this.targetArchive, targetPath, xml);
        });
    }
    /**
     * Copys slide files
     * @internal
     */
    copySlideFiles() {
        return __awaiter(this, void 0, void 0, function* () {
            yield file_helper_1.FileHelper.zipCopy(this.sourceArchive, `ppt/slideMasters/slideMaster${this.sourceNumber}.xml`, this.targetArchive, `ppt/slideMasters/slideMaster${this.targetNumber}.xml`);
            yield file_helper_1.FileHelper.zipCopy(this.sourceArchive, `ppt/slideMasters/_rels/slideMaster${this.sourceNumber}.xml.rels`, this.targetArchive, `ppt/slideMasters/_rels/slideMaster${this.targetNumber}.xml.rels`);
        });
    }
    /**
     * Appends to slide rel
     * @internal
     * @param rootArchive
     * @param relId
     * @param slideCount
     * @returns to slide rel
     */
    appendToSlideRel(rootArchive, relId, slideCount) {
        return xml_helper_1.XmlHelper.append({
            archive: rootArchive,
            file: `ppt/_rels/presentation.xml.rels`,
            parent: (xml) => xml.getElementsByTagName('Relationships')[0],
            tag: 'Relationship',
            attributes: {
                Id: relId,
                Type: `http://schemas.openxmlformats.org/officeDocument/2006/relationships/slideMaster`,
                Target: `slideMasters/slideMaster${slideCount}.xml`,
            },
        });
    }
    /**
     * Appends a new slide to slide list in presentation.xml.
     * If rootArchive has no slides, a new node will be created.
     * "id"-attribute of 'p:sldId'-element must be greater than 255.
     * @internal
     * @param rootArchive
     * @param relId
     * @returns to slide list
     */
    appendToSlideList(rootArchive, relId) {
        return xml_helper_1.XmlHelper.append({
            archive: rootArchive,
            file: `ppt/presentation.xml`,
            parent: (xml) => xml.getElementsByTagName('p:sldMasterIdLst')[0],
            tag: 'p:sldMasterId',
            attributes: {
                'r:id': relId,
            },
        });
    }
    /**
     * Appends slide to content type
     * @internal
     * @param rootArchive
     * @param slideCount
     * @returns slide to content type
     */
    appendSlideMasterToContentType(rootArchive, slideCount) {
        return xml_helper_1.XmlHelper.append(xml_helper_1.XmlHelper.createContentTypeChild(rootArchive, {
            PartName: `/ppt/slideMasters/slideMaster${slideCount}.xml`,
            ContentType: `application/vnd.openxmlformats-officedocument.presentationml.slideMaster+xml`,
        }));
    }
    appendSlideLayoutToContentType(rootArchive, slideLayoutCount) {
        return xml_helper_1.XmlHelper.append(xml_helper_1.XmlHelper.createContentTypeChild(rootArchive, {
            PartName: `/ppt/slideLayouts/slideLayout${slideLayoutCount}.xml`,
            ContentType: `application/vnd.openxmlformats-officedocument.presentationml.slideLayout+xml`,
        }));
    }
    appendThemeToContentType(rootArchive, themeCount) {
        return xml_helper_1.XmlHelper.append(xml_helper_1.XmlHelper.createContentTypeChild(rootArchive, {
            PartName: `/ppt/theme/theme${themeCount}.xml`,
            ContentType: `application/vnd.openxmlformats-officedocument.theme+xml`,
        }));
    }
    /**
     * ToDo: This equals the corresponding method in slide.ts
     *
     * Appends notes to content type
     * @internal
     * @param rootArchive
     * @param slideCount
     * @returns notes to content type
     */
    appendNotesToContentType(rootArchive, slideCount) {
        return xml_helper_1.XmlHelper.append(xml_helper_1.XmlHelper.createContentTypeChild(rootArchive, {
            PartName: `/ppt/notesSlides/notesSlide${slideCount}.xml`,
            ContentType: `application/vnd.openxmlformats-officedocument.presentationml.notesSlide+xml`,
        }));
    }
    /**
     * ToDo: This equals the corresponding method in slide.ts
     *
     * Copys related content
     * @internal
     * @returns related content
     */
    copyRelatedContent() {
        return __awaiter(this, void 0, void 0, function* () {
            const charts = yield chart_1.Chart.getAllOnSlide(this.sourceArchive, this.relsPath);
            for (const chart of charts) {
                yield new chart_1.Chart({
                    mode: 'append',
                    target: chart,
                    sourceArchive: this.sourceArchive,
                    sourceSlideNumber: this.sourceNumber,
                }, this.targetType).modifyOnAddedSlide(this.targetTemplate, this.targetNumber);
            }
            const images = yield image_1.Image.getAllOnSlide(this.sourceArchive, this.relsPath);
            for (const image of images) {
                yield new image_1.Image({
                    mode: 'append',
                    target: image,
                    sourceArchive: this.sourceArchive,
                    sourceSlideNumber: this.sourceNumber,
                }, this.targetType).modifyOnAddedSlide(this.targetTemplate, this.targetNumber);
            }
        });
    }
}
exports.Master = Master;
//# sourceMappingURL=master.js.map